@@ -1,13 +1,12 @@
 
 #' Linear MRCA function
 #'
-#' Function to make the MRCA matrix of a tree, where entry (i,j) gives the MRCA of tips i and j.
+#' Function to make the most recent common ancestor (MRCA) matrix of a tree, where entry (i,j) gives the MRCA of tips i and j.
+#' The function is linear, exploiting the fact that the tree is rooted.
 #'
 #' @author  Michelle Kendall \email{michelle.louise.kendall@@gmail.com}
 #'
-#' @export
-#'
-#' @param tree an object of the class \code{phylo}
+#' @param tree an object of the class \code{phylo} which should be rooted.
 #' @param k (optional) number of tips in tree, for faster computation
 #'
 #' @importFrom phangorn Descendants
@@ -23,7 +22,10 @@
 #' ## create matrix of MRCAs: entry (i,j) is the node number of the MRCA of tips i and j
 #' linearMrca(x,6)
 #'
+#'
+#' @export
 linearMrca <- function(tree,k=0) { # k is number of tips, which can be passed to the function to save on computation
+  if(!is.rooted(tree)){stop("This function requires the tree to be rooted")}
   if (k==0) {k <- length(tree$tip.label)}
   M <- matrix(0, nrow=k, ncol=k); # initialise matrix
   T <- tree$Nnode # total number of internal nodes
@@ -56,40 +58,6 @@ linearMrca <- function(tree,k=0) { # k is number of tips, which can be passed to
 }
 linearMrca <- compiler::cmpfun(linearMrca) # compile
 
-#' Pendant edges
-#'
-#' Extract just the pendant edges from the vector \code{tree$edge}.
-#'
-#' @export
-#'
-#' @author  Michelle Kendall \email{michelle.louise.kendall@@gmail.com}
-#'
-#' @param tree an object of the class \code{phylo}
-#' @param k number of tips in tree
-#'
-#' @importFrom compiler cmpfun
-#'
-penEdgeTree <- function(tree,k) {tree$edge[match(1:k, tree$edge[,2]),] }
-penEdgeTree <- compiler::cmpfun(penEdgeTree)
-
-
-
-#' Pendant edges, matched
-#'
-#' Extract the pendant edges from the vector \code{tree$edge}, in the order given by \code{labelmatch}.
-#'
-#' @export
-#'
-#' @author  Michelle Kendall \email{michelle.louise.kendall@@gmail.com}
-#'
-#' @param tree an object of the class \code{phylo}
-#' @param labelmatch a vector specifying the order of the tips in the output. This is used by other functions in the package to match the tip labels in the trees
-#'
-#' @importFrom compiler cmpfun
-#'
-penEdgeTreematch  <- function(tree,labelmatch) {tree$edge[match(labelmatch, tree$edge[,2]),] }
-penEdgeTreematch <- compiler::cmpfun(penEdgeTreematch)
-
 
 #' Tree vector function
 #'
@@ -97,19 +65,20 @@ penEdgeTreematch <- compiler::cmpfun(penEdgeTreematch)
 #' The elements of the vector are numeric if \code{return.lambda.function=FALSE} (default),
 #' and otherwise they are functions of lambda.
 #'
-#' @export
-#'
 #' @author Jacob Almagro-Garcia \email{nativecoder@@gmail.com}
 #' @author Michelle Kendall \email{michelle.louise.kendall@@gmail.com}
 #'
 #' @param tree an object of the class \code{phylo}
 #' @param lambda a number in [0,1] which specifies the extent to which topology (default, with lambda=0)  or branch lengths (lambda=1) are emphasised. This argument is ignored if \code{return.lambda.function=TRUE}.
 #' @param return.lambda.function If true, a function that can be invoked with different lambda values is returned. This function returns the vector of metric values for the given lambda.
+#' @param emphasise.tips an optional list of tips whose entries in the tree vector should be emphasised. Defaults to \code{NULL}.
+#' @param emphasise.weight applicable only if a list is supplied to \code{emphasise.tips}, this value (default 2) is the number by which vector entries corresponding to those tips are emphasised.
 #'
 #' @return The vector with the metric values or a function that produces the vector given a value of lambda.
 #'
 #' @import ape
 #' @importFrom Rcpp evalCpp
+#' @importFrom combinat combn2
 #' @useDynLib treescape
 #'
 #' @examples
@@ -124,8 +93,10 @@ penEdgeTreematch <- compiler::cmpfun(penEdgeTreematch)
 #' vecAsFunction <- treeVec(tree,return.lambda.function=TRUE)
 #' ## evaluate the vector at lambda=0.5:
 #' vecAsFunction(0.5)
+#' 
 #'
-treeVec <- function(tree, lambda=0, return.lambda.function=FALSE) {
+#' @export
+treeVec <- function(tree, lambda=0, return.lambda.function=FALSE, emphasise.tips=NULL, emphasise.weight=2) {
   if(lambda<0 || lambda>1) stop("Pick lambda in [0,1]")
   if(class(tree)!="phylo") stop("Tree should be of class phylo")
   if(is.rooted(tree)!=TRUE) stop("Metric is for rooted trees only")
@@ -142,11 +113,25 @@ treeVec <- function(tree, lambda=0, return.lambda.function=FALSE) {
 
   # Ordering the edges by first column places the root at the bottom.
   # Descendants will be placed always before parents.
-  edge_order <- order(tree$edge[,1], decreasing=T)
+  edge_order <- ape::reorder.phylo(tree, "postorder", index.only=T)
   edges <- tree$edge[edge_order,]
   edge_lengths <- tree$edge.length[edge_order]
-
-  # We annotated the nodes of the tree in this list. In two passes we are going to
+  
+  # To emphasise the position of certain tips, if needed:
+  if (is.null(emphasise.tips)==FALSE){
+    # translate important tip label names into order
+    emphasise.tips.order <- tip_order[which(tree$tip.label%in%emphasise.tips)]
+    # find the positions where these tips appear in the k choose 2 elements of the final vector
+    pairs <- combn2(1:num_leaves)
+    emphasise.vector.elements <- union(which(pairs[,1]%in%emphasise.tips.order ),which(pairs[,2]%in%emphasise.tips.order ))
+    # make a vector to multiply positions concerning important tips by chosen weighting
+    tip.weighting <- c(sapply(1:length(pairs[,1]), function(x) if(x%in%emphasise.vector.elements){emphasise.weight} else{1}),
+                       sapply(1:num_leaves, function(y) if(y%in%emphasise.tips.order){emphasise.weight} else{1}))
+  }
+  else{tip.weighting <- rep(1,0.5*num_leaves*(num_leaves+1))}
+  
+  
+    # We annotated the nodes of the tree in this list. In two passes we are going to
   # compute the partition each node induces in the tips (bottom-up pass) and the distance
   # (in branch length and number of branches) from the root to each node (top-down pass).
   annotated_nodes <- list()
@@ -251,11 +236,11 @@ treeVec <- function(tree, lambda=0, return.lambda.function=FALSE) {
   })
 
   if(!return.lambda.function)
-    return(lambda * length_root_distances + (1-lambda) * topological_root_distances)
+    return(tip.weighting * (lambda * length_root_distances + (1-lambda) * topological_root_distances))
   else {
     return(function(l) {
       if(l<0 || l>1) stop("Pick lambda in [0,1]")
-      return(l * length_root_distances + (1-l) * topological_root_distances) })
+      return(tip.weighting * (l * length_root_distances + (1-l) * topological_root_distances)) })
   }
 }
 
@@ -266,8 +251,6 @@ treeVec <- function(tree, lambda=0, return.lambda.function=FALSE) {
 #'
 #' Comparison of two trees using the Kendall Colijn metric
 #'
-#' @export
-#'
 #' @author Jacob Almagro-Garcia \email{nativecoder@@gmail.com}
 #' @author Michelle Kendall \email{michelle.louise.kendall@@gmail.com}
 #'
@@ -276,6 +259,9 @@ treeVec <- function(tree, lambda=0, return.lambda.function=FALSE) {
 #' @param lambda a number in [0,1] which specifies the extent to which topology (default, with lambda=0)  or branch lengths (lambda=1) are emphasised. This argument is ignored if \code{type="function"}.
 #' @param return.lambda.function If true, a function that can be invoked with different lambda values is returned.
 #'  This function returns the vector of metric values for the given lambda.
+#' @param emphasise.tips an optional list of tips whose entries in the tree vectors should be emphasised. Defaults to \code{NULL}.
+#' @param emphasise.weight applicable only if a list is supplied to \code{emphasise.tips}, this value (default 2) is the number by which vector entries corresponding to those tips are emphasised.
+#'
 #' @return The vector with the metric values or a function that produces the vector given a value of lambda.
 #'
 #'
@@ -294,15 +280,20 @@ treeVec <- function(tree, lambda=0, return.lambda.function=FALSE) {
 #' ## We can see how the distance changes when moving from focusing on topology to length:
 #' plot(sapply(seq(0,1,length.out=100), function(x) dist.func(x)), type="l",ylab="",xlab="")
 #'
+#' ## The distance may also change if we emphasise the position of certain tips:
+#' plot(sapply(tree.a$tip.label, function(x) treeDist(tree.a,tree.b,emphasise.tips=x)),
+#'      xlab="Tip number",ylab="Distance when  vector entries corresponding to tip are doubled")
 #'
-treeDist <- function(tree.a, tree.b, lambda=0, return.lambda.function=FALSE) {
+#'
+#' @export
+treeDist <- function(tree.a, tree.b, lambda=0, return.lambda.function=FALSE, emphasise.tips=NULL, emphasise.weight=2) {
 
     if(length(tree.a$tip.label) != length(tree.b$tip.label)) stop("Trees must have the same number of tips")
 
     if(setequal(tree.a$tip.label,tree.b$tip.label) == FALSE) stop("Trees must have the same tip label sets")
 
-    metric_a <- treeVec(tree.a, lambda, return.lambda.function)
-    metric_b <- treeVec(tree.b, lambda, return.lambda.function)
+    metric_a <- treeVec(tree.a, lambda, return.lambda.function, emphasise.tips, emphasise.weight)
+    metric_b <- treeVec(tree.b, lambda, return.lambda.function, emphasise.tips, emphasise.weight)
     if(!return.lambda.function) {
         return(sqrt(sum((metric_a - metric_b)^2)))
     }
@@ -318,8 +309,6 @@ treeDist <- function(tree.a, tree.b, lambda=0, return.lambda.function=FALSE) {
 #'
 #' Comparison of a list of trees using the Kendall Colijn metric. Output is given as a pairwise distance matrix. This is equivalent to the \code{$D} output from \code{treescape} but may be preferable for large datasets, and when principal co-ordinate analysis is not required. It includes an option to save memory at the expense of computation time.
 #'
-#' @export
-#'
 #' @author Jacob Almagro-Garcia \email{nativecoder@@gmail.com}
 #' @author Michelle Kendall \email{michelle.louise.kendall@@gmail.com}
 #'
@@ -328,6 +317,9 @@ treeDist <- function(tree.a, tree.b, lambda=0, return.lambda.function=FALSE) {
 #' @param return.lambda.function If true, a function that can be invoked with different lambda values is returned.
 #'  This function returns the matrix of metric values for the given lambda.
 #' @param save.memory A flag that saves a lot of memory but increases the execution time (not compatible with return.lambda.function=TRUE).
+#' @param emphasise.tips an optional list of tips whose entries in the tree vectors should be emphasised. Defaults to \code{NULL}.
+#' @param emphasise.weight applicable only if a list is supplied to \code{emphasise.tips}, this value (default 2) is the number by which vector entries corresponding to those tips are emphasised.
+#'
 #' @return The distance matrix or a function that produces the distance matrix given a value for lambda.
 #'
 #'
@@ -352,10 +344,33 @@ treeDist <- function(tree.a, tree.b, lambda=0, return.lambda.function=FALSE) {
 #' ## save memory by recomputing each tree vector for each pairwise tree comparison (for fixed lambda):
 #' m0.5 <- multiDist(trees,0.5,save.memory=TRUE)
 #'
+#'
+#' @export
 multiDist <- function(trees, lambda=0,
-                      return.lambda.function=FALSE, save.memory=FALSE) {
-
-  num_trees <- length(trees)
+                      return.lambda.function=FALSE, save.memory=FALSE,
+                      emphasise.tips=NULL, emphasise.weight=2) {
+  
+  if(!inherits(trees, "multiPhylo")) stop("trees should be a multiphylo object")
+  num_trees <- length(trees) 
+  if(num_trees<2) {
+    stop("multiDist expects at least two trees")
+  }
+  
+  # make name labels well defined
+  if(is.null(names(trees))) names(trees) <- 1:num_trees 
+  else if(length(unique(names(trees)))!=num_trees){
+    warning("duplicates detected in tree labels - using generic names")
+    names(trees) <- 1:num_trees
+    }
+  lab <- names(trees)
+  
+  # check all trees have same tip labels
+  for (i in 1:num_trees) {
+    if (!setequal(trees[[i]]$tip.label,trees[[1]]$tip.label)) {
+      stop(paste0("Tree ",lab[[i]]," has different tip labels from the first tree."))
+    } 
+  }
+  
 
   # Working with numbers (no functions).
   if(!return.lambda.function) {
@@ -366,7 +381,7 @@ multiDist <- function(trees, lambda=0,
     if(!save.memory) {
 
       # Compute the metric vector for all trees.
-      tree_metrics <- t(sapply(trees, function(tree) {treeVec(tree, lambda, F)}))
+      tree_metrics <- t(sapply(trees, function(tree) {treeVec(tree, lambda, F, emphasise.tips, emphasise.weight)}))
       sapply(1:(num_trees-1), function(i) {
         sapply((i+1):num_trees, function(j) {
           distances[i,j] <<- distances[j,i] <<- sqrt(sum((tree_metrics[i,] - tree_metrics[j,])^2))
@@ -378,7 +393,7 @@ multiDist <- function(trees, lambda=0,
     else {
       sapply(1:(num_trees-1), function(i) {
         sapply((i+1):num_trees, function(j) {
-          distances[i,j] <<- distances[j,i] <<- treeDist(trees[[i]], trees[[j]], lambda, F)
+          distances[i,j] <<- distances[j,i] <<- treeDist(trees[[i]], trees[[j]], lambda, F, emphasise.tips, emphasise.weight)
         })
       })
     }
@@ -393,7 +408,7 @@ multiDist <- function(trees, lambda=0,
       warning("save.memory=TRUE is incompatible with return.lambda.function=TRUE, setting save.memory=FALSE")
 
     # Compute the list of metric functions for all trees.
-    tree_metric_functions <- sapply(trees, function(tree) {treeVec(tree, lambda, T)})
+    tree_metric_functions <- sapply(trees, function(tree) {treeVec(tree, lambda, T, emphasise.tips, emphasise.weight)})
 
     # Inner function that we'll return, computes the distance matrix given lambda.
     compute_distance_matrix_function <- function(l) {
